/*
 * timer_main.c
 *
 *  Created on: 2021/03/23
 *      Author: 0170886
 */
#include <kernel.h>
#include "kernel_cfg.h"
#include <t_syslog.h>
#include <t_stdlib.h>

#include "cup_timer.h"
#include "common.h"
#include "timer.h"

/* 外部参照関数宣言 */
extern void StOffEntry(void);
extern void StOnEntry(void);
extern void StOffEvSw1Off(void);
extern void StOnEvSw1Off(void);
extern void StOffEvSw1On(void);
extern void StOnEvPush1On(void);
extern void StOnEvBaseTime(void);
extern void StOffExit(void);
extern void StOnExit(void);

/*
 *  Glboal Variable
 */
TTM_CTL	gtm_ctl;

/*
 *  Initialization Routine
 */
void
timer_task_init(intptr_t exinf){
	GTM_CTLF = 0;
	GTM_REMTM = 0;
	GTM_NOWSTS = ST_TIMER_OFF;
	GTM_NEXTSTS = ST_TIMER_OFF;
}

/*
 *  Timer Task
 */
/* タイマタスクのジャンプテーブルの定義 */
/* タイマタスクのみEntry,Exitアクションを追加 */
/* Entryアクションの定義 */
static const CALLBACK_TM StateEntry[ST_TIMER_NUM]={
	&StOffEntry	, &StOnEntry
};
/* イベントアクションの定義 */
static const CALLBACK_TM TMStateTable[ST_TIMER_NUM][EV_TIMER_NUM]={
/*   SW1_OFF,           SW1_ON,          PUSH1_ON,         1秒経過 */
	{&StOffEvSw1Off,	&StOffEvSw1On, 	 NULL,			   NULL				},/* タイマOFF状態	*/
    {&StOnEvSw1Off,		NULL,			 &StOnEvPush1On,   &StOnEvBaseTime	},/* タイマON状態	*/
};

/* Exitアクションの定義 */
static const CALLBACK_TM StateExit[ST_TIMER_NUM]={
	&StOffExit	, &StOnExit
};

static bool_t
check_timer_event(void)
{
	bool_t ret;

	/* 他タスクから通知されたイベント番号が範囲内であることを確認 */
	/* uint32_tの為、マイナスは無い */
	if(EV_TIMER_NUM <= GTM_RCVEV) {
		ret = false;
	} else {
		ret = true;
	}
	return(ret);
}

void
timer_task(intptr_t exinf)
{
	const CALLBACK_TM *pFunc;

	DEBUG_SYSLOG(LOG_NOTICE, " Cup Timer 10 start!");

	for (;;) {
        SVC_PERROR(rcv_dtq(TIMER_DTQ, (intptr_t *)&GTM_RCVAD));
		pFunc = &TMStateTable[GTM_NOWSTS][GTM_RCVEV];
        if(true == check_timer_event()) {
        	if(NULL != *pFunc){
        		(*pFunc)();		/* 現状態とイベントに対応する関数を実行 */
        	} else {
        		DEBUG_SYSLOG(LOG_NOTICE, " ignore event. event id = %d ",GTM_RCVEV);
        	}
        	if(GTM_CTLF & DTM_STSCHG) {		/* 状態遷移要求有り? */
        		GTM_CTLF &= ~DTM_STSCHG;	/* 要求クリア		 */
        		(StateExit[GTM_NOWSTS])();	/* 現状態のExit処理実行 */
        		(StateEntry[GTM_NEXTSTS])();/* 次状態のEntry処理実行 */
        		GTM_NOWSTS = GTM_NEXTSTS;   /* 状態更新 		 */
        	}
        	if(GTM_CTLF & DTM_NOTREL){		/* メモリプール解放禁止? */
        		GTM_CTLF &= ~DTM_NOTREL;	/* 要求クリア		 */
        		DEBUG_SYSLOG(LOG_NOTICE, " Not call rel_mpf");
        	}
        	else{
        		DEBUG_SYSLOG(LOG_NOTICE, " rel_mpf = 0x%p", GTM_RCVAD);
        		SVC_PERROR(rel_mpf(EVENT_MPF, GTM_RCVAD));
        	}
        }
        else{
    		DEBUG_SYSLOG(LOG_NOTICE, " rel_mpf = 0x%p", GTM_RCVAD);
    		SVC_PERROR(rel_mpf(EVENT_MPF, GTM_RCVAD));
        }
	}
}
